"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getApiKey = exports.deleteApiKey = exports.saveApiKey = exports.getCustomUrl = exports.saveCustomUrl = exports.decrypt = exports.encrypt = void 0;
const fs = __importStar(require("fs"));
const crypto = __importStar(require("crypto"));
const algorithm = "aes-256-cbc";
const secretKey = "terminalGPT";
/**
 * Encrypts the given text using the specified algorithm, secret key, and initialization vector.
 *
 * @param {string} text - The text to be encrypted.
 * @return {string} The encrypted text in the format: IV:encryptedText.
 */
function encrypt(text) {
    const iv = crypto.randomBytes(16);
    const key = crypto.scryptSync(secretKey, "salt", 32);
    const cipher = crypto.createCipheriv(algorithm, key, iv);
    let encrypted = cipher.update(text);
    encrypted = Buffer.concat([encrypted, cipher.final()]);
    return iv.toString("hex") + ":" + encrypted.toString("hex");
}
exports.encrypt = encrypt;
/**
 * Decrypts the given text using a specific algorithm and secret key.
 *
 * @param {string} text - The text to be decrypted.
 * @return {string} - The decrypted text.
 */
function decrypt(text) {
    const textParts = text.split(":");
    const iv = Buffer.from(textParts.shift(), "hex");
    const encryptedText = Buffer.from(textParts.join(":"), "hex");
    const key = crypto.scryptSync(secretKey, "salt", 32);
    const decipher = crypto.createDecipheriv(algorithm, key, iv);
    let decrypted = decipher.update(encryptedText);
    decrypted = Buffer.concat([decrypted, decipher.final()]);
    return decrypted.toString();
}
exports.decrypt = decrypt;
/**
 * Saves a custom URL to a file.
 *
 * @param {string | undefined} url - The custom URL to be saved. If undefined, the file is deleted.
 * @return {void}
 */
function saveCustomUrl(url) {
    if (url === undefined) {
        fs.unlinkSync(`${__dirname}/customUrl.txt`);
    }
    else {
        fs.writeFileSync(`${__dirname}/customUrl.txt`, url);
    }
}
exports.saveCustomUrl = saveCustomUrl;
/**
 * Retrieves a custom URL from a text file if it exists.
 *
 * @return {string | undefined} The custom URL if it exists, otherwise undefined.
 */
function getCustomUrl() {
    if (fs.existsSync(`${__dirname}/customUrl.txt`)) {
        return fs.readFileSync(`${__dirname}/customUrl.txt`, "utf8");
    }
    return undefined;
}
exports.getCustomUrl = getCustomUrl;
/**
 * Saves the API key to a file.
 *
 * @param {string} apiKey - The API key to save.
 * @return {void} This function does not return anything.
 */
function saveApiKey(apiKey) {
    fs.writeFileSync(`${__dirname}/apiKey.txt`, apiKey);
}
exports.saveApiKey = saveApiKey;
/**
 * Deletes the API key file if it exists.
 *
 * @return {boolean} Returns true if the API key file was deleted, false otherwise.
 */
function deleteApiKey() {
    const apiKeyFilePath = `${__dirname}/apiKey.txt`;
    if (fs.existsSync(apiKeyFilePath)) {
        fs.unlinkSync(apiKeyFilePath);
        return true;
    }
    return false;
}
exports.deleteApiKey = deleteApiKey;
/**
 * Retrieves the API key from the "apiKey.txt" file, decrypts it, and returns it.
 *
 * @return {string | null} The decrypted API key, or null if the file does not exist.
 */
function getApiKey() {
    if (fs.existsSync(`${__dirname}/apiKey.txt`)) {
        const getEncryptedScript = fs.readFileSync(`${__dirname}/apiKey.txt`, "utf8");
        return decrypt(getEncryptedScript);
    }
    return null;
}
exports.getApiKey = getApiKey;
